@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2019, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2021, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -210,6 +210,17 @@ typedef struct dt_field
 
 #define DT_FIELD_NOT_ALLOCATED      1
 
+/*
+ * Structure used for each individual key or value
+ */
+typedef struct dt_table_unit
+{
+    char                    *Value;      /* Field value (from name : value) */
+    UINT32                  Line;        /* Line number for this field */
+    UINT32                  Column;      /* Start column for field value */
+
+} DT_TABLE_UNIT;
+
 
 /*
  * Structure used for individual subtables within an ACPI table
@@ -292,14 +303,6 @@ DtCompilePadding (
     UINT32                  Length,
     DT_SUBTABLE             **RetSubtable);
 
-void
-DtCreateField (
-    char                    *Name,
-    char                    *Value,
-    UINT32                  Line,
-    UINT32                  Offset,
-    UINT32                  Column,
-    UINT32                  NameColumn);
 
 /* dtio - binary and text input/output */
 
@@ -437,9 +440,27 @@ DtCompileFlag (
     ACPI_DMTABLE_INFO       *Info);
 
 
+/* dtfield - DT_FIELD operations */
+
+void
+DtLinkField (
+    DT_FIELD                *Field);
+
+void
+DtCreateField (
+    DT_TABLE_UNIT           *FieldKey,
+    DT_TABLE_UNIT           *FieldValue,
+    UINT32                  Offset);
+
+DT_TABLE_UNIT *
+DtCreateTableUnit (
+    char                    *Data,
+    UINT32                  Line,
+    UINT32                  Column);
+
+
 /* dtparser - lex/yacc files */
 
-UINT64                      DtCompilerParserResult; /* Expression return value */
 int
 DtCompilerParserparse (
     void);
@@ -544,6 +565,10 @@ ACPI_STATUS
 DtCompileRsdp (
     DT_FIELD                **PFieldList);
 
+ACPI_STATUS
+DtCompileAest (
+    void                    **PFieldList);
+
 ACPI_STATUS
 DtCompileAsf (
     void                    **PFieldList);
@@ -552,6 +577,10 @@ ACPI_STATUS
 DtCompileCpep (
     void                    **PFieldList);
 
+ACPI_STATUS
+DtCompileCedt (
+    void                    **PFieldList);
+
 ACPI_STATUS
 DtCompileCsrt (
     void                    **PFieldList);
@@ -625,11 +654,11 @@ DtCompileMsct (
     void                    **PFieldList);
 
 ACPI_STATUS
-DtCompileMtmr (
+DtCompileNfit (
     void                    **PFieldList);
 
 ACPI_STATUS
-DtCompileNfit (
+DtCompileNhlt (
     void                    **PFieldList);
 
 ACPI_STATUS
@@ -640,6 +669,10 @@ ACPI_STATUS
 DtCompilePdtt (
     void                    **PFieldList);
 
+ACPI_STATUS
+DtCompilePhat (
+    void                    **PFieldList);
+
 ACPI_STATUS
 DtCompilePmtt (
     void                    **PFieldList);
@@ -648,6 +681,14 @@ ACPI_STATUS
 DtCompilePptt (
     void                    **PFieldList);
 
+ACPI_STATUS
+DtCompilePrmt (
+    void                    **PFieldList);
+
+ACPI_STATUS
+DtCompileRgrt (
+    void                    **PFieldList);
+
 ACPI_STATUS
 DtCompileRsdt (
     void                    **PFieldList);
@@ -676,6 +717,10 @@ ACPI_STATUS
 DtCompileStao (
     void                    **PFieldList);
 
+ACPI_STATUS
+DtCompileSvkl (
+    void                    **PFieldList);
+
 ACPI_STATUS
 DtCompileTcpa (
     void                    **PFieldList);
@@ -689,7 +734,7 @@ DtCompileUefi (
     void                    **PFieldList);
 
 ACPI_STATUS
-DtCompileVrtc (
+DtCompileViot (
     void                    **PFieldList);
 
 ACPI_STATUS
@@ -716,10 +761,13 @@ DtGetGenericTableInfo (
 
 /* ACPI Table templates */
 
+extern const unsigned char  TemplateAest[];
 extern const unsigned char  TemplateAsf[];
 extern const unsigned char  TemplateBoot[];
+extern const unsigned char  TemplateBdat[];
 extern const unsigned char  TemplateBert[];
 extern const unsigned char  TemplateBgrt[];
+extern const unsigned char  TemplateCedt[];
 extern const unsigned char  TemplateCpep[];
 extern const unsigned char  TemplateCsrt[];
 extern const unsigned char  TemplateDbg2[];
@@ -744,13 +792,15 @@ extern const unsigned char  TemplateMchi[];
 extern const unsigned char  TemplateMpst[];
 extern const unsigned char  TemplateMsct[];
 extern const unsigned char  TemplateMsdm[];
-extern const unsigned char  TemplateMtmr[];
 extern const unsigned char  TemplateNfit[];
 extern const unsigned char  TemplatePcct[];
 extern const unsigned char  TemplatePdtt[];
+extern const unsigned char  TemplatePhat[];
 extern const unsigned char  TemplatePmtt[];
 extern const unsigned char  TemplatePptt[];
+extern const unsigned char  TemplatePrmt[];
 extern const unsigned char  TemplateRasf[];
+extern const unsigned char  TemplateRgrt[];
 extern const unsigned char  TemplateRsdt[];
 extern const unsigned char  TemplateS3pt[];
 extern const unsigned char  TemplateSbst[];
@@ -762,10 +812,11 @@ extern const unsigned char  TemplateSpcr[];
 extern const unsigned char  TemplateSpmi[];
 extern const unsigned char  TemplateSrat[];
 extern const unsigned char  TemplateStao[];
+extern const unsigned char  TemplateSvkl[];
 extern const unsigned char  TemplateTcpa[];
 extern const unsigned char  TemplateTpm2[];
 extern const unsigned char  TemplateUefi[];
-extern const unsigned char  TemplateVrtc[];
+extern const unsigned char  TemplateViot[];
 extern const unsigned char  TemplateWaet[];
 extern const unsigned char  TemplateWdat[];
 extern const unsigned char  TemplateWddt[];
