@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2015, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2021, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -111,10 +111,45 @@
  * other governmental approval, or letter of assurance, without first obtaining
  * such license, approval or letter.
  *
+ *****************************************************************************
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * following license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions, and the following disclaimer,
+ *    without modification.
+ * 2. Redistributions in binary form must reproduce at minimum a disclaimer
+ *    substantially similar to the "NO WARRANTY" disclaimer below
+ *    ("Disclaimer") and any redistribution must be conditioned upon
+ *    including a substantially similar Disclaimer requirement for further
+ *    binary redistribution.
+ * 3. Neither the names of the above-listed copyright holders nor the names
+ *    of any contributors may be used to endorse or promote products derived
+ *    from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * GNU General Public License ("GPL") version 2 as published by the Free
+ * Software Foundation.
+ *
  *****************************************************************************/
 
 #include "acpidump.h"
-#include "acapps.h"
 
 
 /* Local prototypes */
@@ -124,26 +159,45 @@ ApIsExistingFile (
     char                    *Pathname);
 
 
+/******************************************************************************
+ *
+ * FUNCTION:    ApIsExistingFile
+ *
+ * PARAMETERS:  Pathname            - Output filename
+ *
+ * RETURN:      0 on success
+ *
+ * DESCRIPTION: Query for file overwrite if it already exists.
+ *
+ ******************************************************************************/
+
 static int
 ApIsExistingFile (
     char                    *Pathname)
 {
-#ifndef _GNU_EFI
+#if !defined(_GNU_EFI) && !defined(_EDK2_EFI)
     struct stat             StatInfo;
+    int                     InChar;
 
 
     if (!stat (Pathname, &StatInfo))
     {
-        AcpiLogError ("Target path already exists, overwrite? [y|n] ");
+        fprintf (stderr, "Target path already exists, overwrite? [y|n] ");
 
-        if (getchar () != 'y')
+        InChar = fgetc (stdin);
+        if (InChar == '\n')
+        {
+            InChar = fgetc (stdin);
+        }
+
+        if (InChar != 'y' && InChar != 'Y')
         {
             return (-1);
         }
     }
 #endif
 
-    return 0;
+    return (0);
 }
 
 
@@ -176,10 +230,10 @@ ApOpenOutputFile (
 
     /* Point stdout to the file */
 
-    File = AcpiOsOpenFile (Pathname, ACPI_FILE_WRITING);
+    File = fopen (Pathname, "w");
     if (!File)
     {
-        AcpiLogError ("Could not open output file: %s\n", Pathname);
+        fprintf (stderr, "Could not open output file: %s\n", Pathname);
         return (-1);
     }
 
@@ -210,10 +264,10 @@ ApWriteToBinaryFile (
     ACPI_TABLE_HEADER       *Table,
     UINT32                  Instance)
 {
-    char                    Filename[ACPI_NAME_SIZE + 16];
+    char                    Filename[ACPI_NAMESEG_SIZE + 16];
     char                    InstanceStr [16];
     ACPI_FILE               File;
-    size_t                  Actual;
+    ACPI_SIZE               Actual;
     UINT32                  TableLength;
 
 
@@ -225,23 +279,24 @@ ApWriteToBinaryFile (
 
     if (ACPI_VALIDATE_RSDP_SIG (Table->Signature))
     {
-        ACPI_MOVE_NAME (Filename, ACPI_RSDP_NAME);
+        ACPI_COPY_NAMESEG (Filename, ACPI_RSDP_NAME);
     }
     else
     {
-        ACPI_MOVE_NAME (Filename, Table->Signature);
+        ACPI_COPY_NAMESEG (Filename, Table->Signature);
     }
+
     Filename[0] = (char) tolower ((int) Filename[0]);
     Filename[1] = (char) tolower ((int) Filename[1]);
     Filename[2] = (char) tolower ((int) Filename[2]);
     Filename[3] = (char) tolower ((int) Filename[3]);
-    Filename[ACPI_NAME_SIZE] = 0;
+    Filename[ACPI_NAMESEG_SIZE] = 0;
 
     /* Handle multiple SSDTs - create different filenames for each */
 
     if (Instance > 0)
     {
-        AcpiUtSnprintf (InstanceStr, sizeof (InstanceStr), "%u", Instance);
+        snprintf (InstanceStr, sizeof (InstanceStr), "%u", Instance);
         strcat (Filename, InstanceStr);
     }
 
@@ -249,30 +304,29 @@ ApWriteToBinaryFile (
 
     if (Gbl_VerboseMode)
     {
-        AcpiLogError (
+        fprintf (stderr,
             "Writing [%4.4s] to binary file: %s 0x%X (%u) bytes\n",
             Table->Signature, Filename, Table->Length, Table->Length);
     }
 
     /* Open the file and dump the entire table in binary mode */
 
-    File = AcpiOsOpenFile (Filename,
-        ACPI_FILE_WRITING | ACPI_FILE_BINARY);
+    File = fopen (Filename, "wb");
     if (!File)
     {
-        AcpiLogError ("Could not open output file: %s\n", Filename);
+        fprintf (stderr, "Could not open output file: %s\n", Filename);
         return (-1);
     }
 
-    Actual = AcpiOsWriteFile (File, Table, 1, TableLength);
+    Actual = fwrite (Table, 1, TableLength, File);
     if (Actual != TableLength)
     {
-        AcpiLogError ("Error writing binary output file: %s\n", Filename);
-        AcpiOsCloseFile (File);
+        fprintf (stderr, "Error writing binary output file: %s\n", Filename);
+        fclose (File);
         return (-1);
     }
 
-    AcpiOsCloseFile (File);
+    fclose (File);
     return (0);
 }
 
@@ -298,15 +352,15 @@ ApGetTableFromFile (
     ACPI_TABLE_HEADER       *Buffer = NULL;
     ACPI_FILE               File;
     UINT32                  FileSize;
-    size_t                  Actual;
+    ACPI_SIZE               Actual;
 
 
     /* Must use binary mode */
 
-    File = AcpiOsOpenFile (Pathname, ACPI_FILE_READING | ACPI_FILE_BINARY);
+    File = fopen (Pathname, "rb");
     if (!File)
     {
-        AcpiLogError ("Could not open input file: %s\n", Pathname);
+        fprintf (stderr, "Could not open input file: %s\n", Pathname);
         return (NULL);
     }
 
@@ -315,7 +369,7 @@ ApGetTableFromFile (
     FileSize = CmGetFileSize (File);
     if (FileSize == ACPI_UINT32_MAX)
     {
-        AcpiLogError (
+        fprintf (stderr,
             "Could not get input file size: %s\n", Pathname);
         goto Cleanup;
     }
@@ -325,18 +379,17 @@ ApGetTableFromFile (
     Buffer = ACPI_ALLOCATE_ZEROED (FileSize);
     if (!Buffer)
     {
-        AcpiLogError (
+        fprintf (stderr,
             "Could not allocate file buffer of size: %u\n", FileSize);
         goto Cleanup;
     }
 
     /* Read the entire file */
 
-    Actual = AcpiOsReadFile (File, Buffer, 1, FileSize);
+    Actual = fread (Buffer, 1, FileSize, File);
     if (Actual != FileSize)
     {
-        AcpiLogError (
-            "Could not read input file: %s\n", Pathname);
+        fprintf (stderr, "Could not read input file: %s\n", Pathname);
         ACPI_FREE (Buffer);
         Buffer = NULL;
         goto Cleanup;
@@ -345,6 +398,6 @@ ApGetTableFromFile (
     *OutFileSize = FileSize;
 
 Cleanup:
-    AcpiOsCloseFile (File);
+    fclose (File);
     return (Buffer);
 }
