@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2013, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2021, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -111,11 +111,46 @@
  * other governmental approval, or letter of assurance, without first obtaining
  * such license, approval or letter.
  *
+ *****************************************************************************
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * following license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions, and the following disclaimer,
+ *    without modification.
+ * 2. Redistributions in binary form must reproduce at minimum a disclaimer
+ *    substantially similar to the "NO WARRANTY" disclaimer below
+ *    ("Disclaimer") and any redistribution must be conditioned upon
+ *    including a substantially similar Disclaimer requirement for further
+ *    binary redistribution.
+ * 3. Neither the names of the above-listed copyright holders nor the names
+ *    of any contributors may be used to endorse or promote products derived
+ *    from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * GNU General Public License ("GPL") version 2 as published by the Free
+ * Software Foundation.
+ *
  *****************************************************************************/
 
 #define _DECLARE_GLOBALS
 #include "acpidump.h"
-#include "acapps.h"
 
 
 /*
@@ -151,7 +186,7 @@ ApDoOptions (
     int                     argc,
     char                    **argv);
 
-static void
+static int
 ApInsertAction (
     char                    *Argument,
     UINT32                  ToBeDone);
@@ -164,7 +199,7 @@ UINT32                      CurrentAction = 0;
 
 
 #define AP_UTILITY_NAME             "ACPI Binary Table Dump Utility"
-#define AP_SUPPORTED_OPTIONS        "?a:bcf:hn:o:r:svz"
+#define AP_SUPPORTED_OPTIONS        "?a:bc:f:hn:o:r:sv^xz"
 
 
 /******************************************************************************
@@ -183,21 +218,23 @@ ApDisplayUsage (
     ACPI_USAGE_HEADER ("acpidump [options]");
 
     ACPI_OPTION ("-b",                      "Dump tables to binary files");
-    ACPI_OPTION ("-c",                      "Dump customized tables");
     ACPI_OPTION ("-h -?",                   "This help message");
     ACPI_OPTION ("-o <File>",               "Redirect output to file");
     ACPI_OPTION ("-r <Address>",            "Dump tables from specified RSDP");
     ACPI_OPTION ("-s",                      "Print table summaries only");
     ACPI_OPTION ("-v",                      "Display version information");
+    ACPI_OPTION ("-vd",                     "Display build date and time");
     ACPI_OPTION ("-z",                      "Verbose mode");
 
-    printf ("\nTable Options:\n");
+    ACPI_USAGE_TEXT ("\nTable Options:\n");
 
     ACPI_OPTION ("-a <Address>",            "Get table via a physical address");
+    ACPI_OPTION ("-c <on|off>",             "Turning on/off customized table dumping");
     ACPI_OPTION ("-f <BinaryFile>",         "Get table via a binary file");
     ACPI_OPTION ("-n <Signature>",          "Get table via a name/signature");
+    ACPI_OPTION ("-x",                      "Use RSDT instead of XSDT");
 
-    printf (
+    ACPI_USAGE_TEXT (
         "\n"
         "Invocation without parameters dumps all available tables\n"
         "Multiple mixed instances of -a, -f, and -n are supported\n\n");
@@ -211,13 +248,13 @@ ApDisplayUsage (
  * PARAMETERS:  Argument            - Pointer to the argument for this action
  *              ToBeDone            - What to do to process this action
  *
- * RETURN:      None. Exits program if action table becomes full.
+ * RETURN:      Status
  *
  * DESCRIPTION: Add an action item to the action table
  *
  ******************************************************************************/
 
-static void
+static int
 ApInsertAction (
     char                    *Argument,
     UINT32                  ToBeDone)
@@ -231,9 +268,11 @@ ApInsertAction (
     CurrentAction++;
     if (CurrentAction > AP_MAX_ACTIONS)
     {
-        fprintf (stderr, "Too many table options (max %u)\n", AP_MAX_ACTIONS);
-        exit (-1);
+        fprintf (stderr, "Too many table options (max %d)\n", AP_MAX_ACTIONS);
+        return (-1);
     }
+
+    return (0);
 }
 
 
@@ -261,7 +300,7 @@ ApDoOptions (
 
     /* Command line options */
 
-    while ((j = AcpiGetopt (argc, argv, AP_SUPPORTED_OPTIONS)) != EOF) switch (j)
+    while ((j = AcpiGetopt (argc, argv, AP_SUPPORTED_OPTIONS)) != ACPI_OPT_END) switch (j)
     {
     /*
      * Global options
@@ -273,31 +312,44 @@ ApDoOptions (
 
     case 'c':   /* Dump customized tables */
 
-        Gbl_DumpCustomizedTables = TRUE;
+        if (!strcmp (AcpiGbl_Optarg, "on"))
+        {
+            Gbl_DumpCustomizedTables = TRUE;
+        }
+        else if (!strcmp (AcpiGbl_Optarg, "off"))
+        {
+            Gbl_DumpCustomizedTables = FALSE;
+        }
+        else
+        {
+            fprintf (stderr, "%s: Cannot handle this switch, please use on|off\n",
+                AcpiGbl_Optarg);
+            return (-1);
+        }
         continue;
 
     case 'h':
     case '?':
 
         ApDisplayUsage ();
-        exit (0);
+        return (1);
 
     case 'o':   /* Redirect output to a single file */
 
         if (ApOpenOutputFile (AcpiGbl_Optarg))
         {
-            exit (-1);
+            return (-1);
         }
         continue;
 
     case 'r':   /* Dump tables from specified RSDP */
 
-        Status = AcpiUtStrtoul64 (AcpiGbl_Optarg, 0, &Gbl_RsdpBase);
+        Status = AcpiUtStrtoul64 (AcpiGbl_Optarg, &Gbl_RsdpBase);
         if (ACPI_FAILURE (Status))
         {
             fprintf (stderr, "%s: Could not convert to a physical address\n",
                 AcpiGbl_Optarg);
-            exit (-1);
+            return (-1);
         }
         continue;
 
@@ -306,10 +358,39 @@ ApDoOptions (
         Gbl_SummaryMode = TRUE;
         continue;
 
-    case 'v':   /* Revision/version */
+    case 'x':   /* Do not use XSDT */
 
-        printf (ACPI_COMMON_SIGNON (AP_UTILITY_NAME));
-        exit (0);
+        if (!AcpiGbl_DoNotUseXsdt)
+        {
+            AcpiGbl_DoNotUseXsdt = TRUE;
+        }
+        else
+        {
+            Gbl_DoNotDumpXsdt = TRUE;
+        }
+        continue;
+
+    case 'v': /* -v: (Version): signon already emitted, just exit */
+
+        switch (AcpiGbl_Optarg[0])
+        {
+        case '^':  /* -v: (Version) */
+
+            fprintf (stderr, ACPI_COMMON_SIGNON (AP_UTILITY_NAME));
+            return (1);
+
+        case 'd':
+
+            fprintf (stderr, ACPI_COMMON_SIGNON (AP_UTILITY_NAME));
+            printf (ACPI_COMMON_BUILD_TIME);
+            return (1);
+
+        default:
+
+            printf ("Unknown option: -v%s\n", AcpiGbl_Optarg);
+            return (-1);
+        }
+        break;
 
     case 'z':   /* Verbose mode */
 
@@ -322,30 +403,42 @@ ApDoOptions (
      */
     case 'a':   /* Get table by physical address */
 
-        ApInsertAction (AcpiGbl_Optarg, AP_DUMP_TABLE_BY_ADDRESS);
+        if (ApInsertAction (AcpiGbl_Optarg, AP_DUMP_TABLE_BY_ADDRESS))
+        {
+            return (-1);
+        }
         break;
 
     case 'f':   /* Get table from a file */
 
-        ApInsertAction (AcpiGbl_Optarg, AP_DUMP_TABLE_BY_FILE);
+        if (ApInsertAction (AcpiGbl_Optarg, AP_DUMP_TABLE_BY_FILE))
+        {
+            return (-1);
+        }
         break;
 
     case 'n':   /* Get table by input name (signature) */
 
-        ApInsertAction (AcpiGbl_Optarg, AP_DUMP_TABLE_BY_NAME);
+        if (ApInsertAction (AcpiGbl_Optarg, AP_DUMP_TABLE_BY_NAME))
+        {
+            return (-1);
+        }
         break;
 
     default:
 
         ApDisplayUsage ();
-        exit (-1);
+        return (-1);
     }
 
     /* If there are no actions, this means "get/dump all tables" */
 
     if (CurrentAction == 0)
     {
-        ApInsertAction (NULL, AP_DUMP_ALL_TABLES);
+        if (ApInsertAction (NULL, AP_DUMP_ALL_TABLES))
+        {
+            return (-1);
+        }
     }
 
     return (0);
@@ -364,10 +457,17 @@ ApDoOptions (
  *
  ******************************************************************************/
 
+#if !defined(_GNU_EFI) && !defined(_EDK2_EFI)
 int ACPI_SYSTEM_XFACE
 main (
     int                     argc,
     char                    *argv[])
+#else
+int ACPI_SYSTEM_XFACE
+acpi_main (
+    int                     argc,
+    char                    *argv[])
+#endif
 {
     int                     Status = 0;
     AP_DUMP_ACTION          *Action;
@@ -376,12 +476,20 @@ main (
 
 
     ACPI_DEBUG_INITIALIZE (); /* For debug version only */
+    AcpiOsInitialize ();
+    Gbl_OutputFile = ACPI_FILE_OUT;
+    AcpiGbl_IntegerByteWidth = 8;
 
     /* Process command line options */
 
-    if (ApDoOptions (argc, argv))
+    Status = ApDoOptions (argc, argv);
+    if (Status > 0)
     {
-        return (-1);
+        return (0);
+    }
+    if (Status < 0)
+    {
+        return (Status);
     }
 
     /* Get/dump ACPI table(s) as requested */
@@ -424,13 +532,13 @@ main (
         }
     }
 
-    if (Gbl_OutputFile)
+    if (Gbl_OutputFilename)
     {
         if (Gbl_VerboseMode)
         {
             /* Summary for the output file */
 
-            FileSize = ApGetFileSize (Gbl_OutputFile);
+            FileSize = CmGetFileSize (Gbl_OutputFile);
             fprintf (stderr, "Output file %s contains 0x%X (%u) bytes\n\n",
                 Gbl_OutputFilename, FileSize, FileSize);
         }
