@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2019, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2021, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -190,6 +190,7 @@ AeMiscellaneousTests (
     ACPI_STATUS             Status;
     ACPI_STATISTICS         Stats;
     ACPI_HANDLE             Handle;
+    UINT32                  TableIndex;
 
 #if (!ACPI_REDUCED_HARDWARE)
     UINT32                  Temp;
@@ -218,18 +219,15 @@ AeMiscellaneousTests (
 
         /* Load and unload SSDT4 */
 
-        Status = AcpiLoadTable ((ACPI_TABLE_HEADER *) Ssdt4Code);
+        Status = AcpiLoadTable ((ACPI_TABLE_HEADER *) Ssdt4Code, &TableIndex);
         ACPI_CHECK_OK (AcpiLoadTable, Status);
 
-        Status = AcpiGetHandle (NULL, "\\_T96", &Handle);
-        ACPI_CHECK_OK (AcpiGetHandle, Status);
-
-        Status = AcpiUnloadParentTable (Handle);
-        ACPI_CHECK_OK (AcpiUnloadParentTable, Status);
+        Status = AcpiUnloadTable (TableIndex);
+        ACPI_CHECK_OK (AcpiUnloadTable, Status);
 
         /* Re-load SSDT4 */
 
-        Status = AcpiLoadTable ((ACPI_TABLE_HEADER *) Ssdt4Code);
+        Status = AcpiLoadTable ((ACPI_TABLE_HEADER *) Ssdt4Code, NULL);
         ACPI_CHECK_OK (AcpiLoadTable, Status);
 
         /* Unload and re-load SSDT2 (SSDT2 is in the XSDT) */
@@ -240,12 +238,12 @@ AeMiscellaneousTests (
         Status = AcpiUnloadParentTable (Handle);
         ACPI_CHECK_OK (AcpiUnloadParentTable, Status);
 
-        Status = AcpiLoadTable ((ACPI_TABLE_HEADER *) Ssdt2Code);
+        Status = AcpiLoadTable ((ACPI_TABLE_HEADER *) Ssdt2Code, NULL);
         ACPI_CHECK_OK (AcpiLoadTable, Status);
 
         /* Load OEM9 table (causes table override) */
 
-        Status = AcpiLoadTable ((ACPI_TABLE_HEADER *) Ssdt3Code);
+        Status = AcpiLoadTable ((ACPI_TABLE_HEADER *) Ssdt3Code, NULL);
         ACPI_CHECK_OK (AcpiLoadTable, Status);
     }
 
